//  cspace.cpp
//  Created by Rafael Guerrero on 4/16/12.
//  Copyright (c) 2012 University of Texas. All rights reserved.
//

//  The 4 continuous space models used in Ayala et al (2012) Evolution, part of the inv2lafit program

#include <iostream>
using std::cout;
using std::endl;

#include <fstream>
using std::ifstream;
using std::ofstream;

#include <sstream>
using std::stringstream;

#include <iomanip>
using std::setw;
using std::setprecision;

#include <string>
using std::string;

#include<math.h>

#include <algorithm>
using std::max;

#include<vector>
using std::vector;

#include <map>
using std::map;

#include <boost/math/distributions/normal.hpp> // for normal_distribution
using boost::math::normal; 
#include <boost/progress.hpp>
using boost::progress_timer;
#include <boost/timer.hpp>
using boost::timer;

#include "amebsa.h"
#include "population.h"
#include "utils.h"
#include "init_commons.h"


Doub CthreeFnot (VecDoub_I & vec ) {
    Doub ret; 
    Doub penalty; 
    double epsilon_one_run;
    int maxGens_one_run;
    
	SETRUNPARS
    
    int N_migpoints;
	int N_lpoints;
    SET_CSPACE_PARS
    
    // vec 0 thru 5 are the fitness parameters, go into the Fitness vector
    vector< Genotype > W;
	Genotype    dumW0 ={vec[0], 1, vec[1]}; W.push_back(dumW0);   
    Genotype    dumW1 ={vec[2], 1, vec[3]}; W.push_back(dumW1); 
    Genotype    dumW2 ={vec[4], 1, vec[5]}; W.push_back(dumW2); 
      // vec6 is the assortative mating parameter, goes into F
    double F= 0;  
	//Assigning vec 7 to the migration kernel Standard deviation
    double sigma = vec[6];
	double borderSH= vec[7]; 
    double borderHR= vec[8];

	
	
	// bounds for fitness
	for(int i=0;i<6;++i){
		if(vec[i]< 0)return penalty * (1.0+fabs(vec[i]));
		if(vec[i]>10000) return penalty * (1.0+fabs(vec[i]));
	}
	
    // bounds for migration distance
	if(vec[6]<0.5 || vec[6] > 50) return penalty * (1.0+fabs(vec[7]));
	
	//bounds for region Borders
	if(borderSH  < 206 || borderSH > 340) return penalty ;
	if(borderHR  < 523 || borderHR > 680) return penalty ;

   
  	
	
    int nPops;
    vector<double> ss ;
    vector<double> si ;
    vector<double> ii ; 
    vector<double> distances ; 
    vector<double> latitude ; 

    INPUTDATA	
	INPUT_LATITUDE
    
  	//passing it to a vector, finding the southmost point
    vector<double> pop_positions; 
	double southpoint=0;
    for (int k=0; k < nPops; ++k ) { pop_positions.push_back(latitude[k]); if (southpoint < latitude[k]) southpoint = latitude[k];}
    
    
    // This one is to get the average frequencies in a region. First, we get the counts per region (initCounts)
    vector< vector<double> > initCounts;
    vector<double> zeroDum (3,0) ; for (int i =0; i<3; ++i) initCounts.push_back(zeroDum);
    
    for (int k=0; k < nPops; ++k ) {          
        if (pop_positions.at(k) < borderSH) {
            initCounts.at(0)[0]+= ss[k]; initCounts.at(0)[1]+= si[k]; initCounts.at(0)[2]+= ii[k];
        }
        else if (pop_positions.at(k) < borderHR){
            initCounts.at(1)[0]+= ss[k]; initCounts.at(1)[1]+= si[k]; initCounts.at(1)[2]+= ii[k];
        }
        else {
            initCounts.at(2)[0]+= ss[k]; initCounts.at(2)[1]+= si[k]; initCounts.at(2)[2]+= ii[k];
        }
    }
    
    //Now we pass them to a 3x3 matrix of genotype frequencies per region (initFreq)
	vector<Genotype> initFreq;
    for (int r=0; r<3; ++r) {
		
        double initSS = max( 1.0, initCounts.at(r)[0] )/ (max( 1.0, initCounts.at(r)[0])+ max( 1.0, initCounts.at(r)[1]) + max( 1.0, initCounts.at(r)[2]));
		double initSI = max( 1.0, initCounts.at(r)[1] )/ (max( 1.0, initCounts.at(r)[0])+ max( 1.0, initCounts.at(r)[1]) + max( 1.0, initCounts.at(r)[2]));
		double initII = max( 1.0, initCounts.at(r)[2] )/ (max( 1.0, initCounts.at(r)[0])+ max( 1.0, initCounts.at(r)[1]) + max( 1.0, initCounts.at(r)[2]));
        
		Genotype dumGeno = {initSS,initSI, initII};
        initFreq.push_back(dumGeno);
		
	}
    initFreq.at(0) = deterministic(vec[0], vec[1]);
	initFreq.at(1) = deterministic(vec[2], vec[3]);
	initFreq.at(2) = deterministic(vec[4], vec[5]);

 
	double lattice_h = southpoint / ((double) N_lpoints);
    int SH_limitpoint = ceil(borderSH / lattice_h);                 // gives the first lattice point past the change of region
    int HR_limitpoint = ceil(borderHR / lattice_h);
	
    
	// We will use this vector of populations in the simulation. We will fill it up with initial frequencies
    vector<Population> lat;
    
    for(int i=0; i<N_lpoints+1; ++i){
        if          (i< SH_limitpoint)  { Population newpop(initFreq[0]); lat.push_back(newpop);}
        else if     (i < HR_limitpoint) { Population newpop(initFreq[1]); lat.push_back(newpop);}
        else                            { Population newpop(initFreq[2]); lat.push_back(newpop);}
    }
   	
   	//create a normal distribution from which we will calculate the migration kernel values
	normal dispersion(0, sigma);
	
	//adjust the desired sampling of the migration kernel to the distances of the lattice
	double maxDistance= 3*sigma;
	int nPts_side=((double)N_migpoints-1.0)*0.5;
	int targetPoint = ceil(maxDistance / lattice_h);
	int adjustedPoint = targetPoint + (nPts_side - targetPoint%nPts_side);
	int step = adjustedPoint / nPts_side;
	
    vector<double> mig_prob;
	vector<int> mig_points;
	
  	//We start at the leftmost end of the migration kernel point
    double k= -(double)adjustedPoint*lattice_h ;
	int s = -adjustedPoint; 
    // Filling in the set of points that will be in the migration kernel

	for(int m=0; m< N_migpoints; ++m){
		double kernEval = k+(double)step*lattice_h*m;
        double prob = pdf(dispersion, kernEval);
		mig_prob.push_back(prob);
		int score = s+ step*m;
		mig_points.push_back(score);
	
	}
	
	// RUN!
	simulate(lat, mig_prob, mig_points, W, F, epsilon_one_run, maxGens_one_run, SH_limitpoint, HR_limitpoint, lattice_h);	
    
    vector<Genotype> interpol; 
	
	for (int i=0; i < nPops; ++i){
        int rightpoint = ceil(pop_positions.at(i) / lattice_h );   // lattice point to the right of the current pop 
        int leftpoint = max(1,rightpoint) -1;
        double gain = pop_positions.at(i) - (double)leftpoint*lattice_h;
        
        double SStemp = interpolate(lat.at(leftpoint).getAdults().SS, lat.at(rightpoint).getAdults().SS, gain, lattice_h);
        double SItemp = interpolate(lat.at(leftpoint).getAdults().SI, lat.at(rightpoint).getAdults().SI, gain, lattice_h);
        double IItemp = interpolate(lat.at(leftpoint).getAdults().II, lat.at(rightpoint).getAdults().II, gain, lattice_h);
        Genotype temp ={SStemp, SItemp, IItemp};
        if(SStemp+SItemp+IItemp >1.0) {
            Genotype fixtemp={SStemp/(SStemp+SItemp+IItemp), SItemp/(SStemp+SItemp+IItemp), IItemp/(SStemp+SItemp+IItemp)};
            temp=fixtemp;
        }  
        interpol.push_back(temp);
    }
    
	
    for (int k=0; k< nPops ; ++k ) {
		
        if(ss[k]!=0 && log(interpol.at(k).SS) != 0){ret += -ss[k]* max(log(interpol.at(k).SS), -penalty);}
 		if(si[k]!=0 && log(interpol.at(k).SI) != 0){ret += -si[k]* max(log(interpol.at(k).SI), -penalty);}
 		if(ii[k]!=0 && log(interpol.at(k).II) != 0){ret += -ii[k]* max(log(interpol.at(k).II), -penalty);}	
		
    }
	
    if(ret==NaN || ret!=ret ) { // according to IEEE standards, n!=n when n it is nan
		cout<<"WARNING->likelihood=NaN"<<endl;
		
		return penalty ;
	}
	return ret ;
	
	
	
}
Doub cthree (VecDoub_I & vec ) {
    Doub ret; 
    Doub penalty; 
    double epsilon_one_run;
    int maxGens_one_run;
    
	SETRUNPARS
    
    int N_migpoints;
	int N_lpoints;
    SET_CSPACE_PARS
    
    // vec 0 thru 5 are the fitness parameters, go into the Fitness vector
    vector< Genotype > W;
	Genotype    dumW0 ={vec[0], 1, vec[1]}; W.push_back(dumW0);   
    Genotype    dumW1 ={vec[2], 1, vec[3]}; W.push_back(dumW1); 
    Genotype    dumW2 ={vec[4], 1, vec[5]}; W.push_back(dumW2); 
    // vec6 is the assortative mating parameter, goes into F
    double F= vec[6];  
	//Assigning vec 7 to the migration kernel Standard deviation
    double sigma = vec[7];
	double borderSH= vec[8]; 
    double borderHR= vec[9];
    
	
	// bounds for fitness
	for(int i=0;i<6;++i){
		if(vec[i]< 0)return penalty * (1.0+fabs(vec[i]));
		if(vec[i]>10000) return penalty * (1.0+fabs(vec[i]));
	}
	// bounds for F
	if(vec[6]>1||vec[6]<0) return penalty * (1.0+fabs(vec[6]));
	
    // bounds for migration distance
	if(vec[7]<0.5 || vec[7] > 50) return penalty * (1.0+fabs(vec[7]));
	
	//bounds for region Borders
	if(borderSH  < 206 || borderSH > 340) return penalty ;
	if(borderHR  < 523 || borderHR > 680) return penalty ;
    
    
    
    //************ DA hard coded, ugh, observed counts of mosquitoes per population
    // number of populations sampled
    int nPops;
    vector<double> ss ;
    vector<double> si ;
    vector<double> ii ; 
    vector<double> distances ; 
    vector<double> latitude ; 
    
    INPUTDATA	
	INPUT_LATITUDE
    
    
	//passing it to a vector, finding the southmost point
    vector<double> pop_positions; 
	double southpoint=0;
    for (int k=0; k < nPops; ++k ) { pop_positions.push_back(latitude[k]); if (southpoint < latitude[k]) southpoint = latitude[k];}
    
    
    // This one is to get the average frequencies in a region. First, we get the counts per region (initCounts)
    vector< vector<double> > initCounts;
    vector<double> zeroDum (3,0) ; for (int i =0; i<3; ++i) initCounts.push_back(zeroDum);
    
    for (int k=0; k < nPops; ++k ) {          
        if (pop_positions.at(k) < borderSH) {
            initCounts.at(0)[0]+= ss[k]; initCounts.at(0)[1]+= si[k]; initCounts.at(0)[2]+= ii[k];
        }
        else if (pop_positions.at(k) < borderHR){
            initCounts.at(1)[0]+= ss[k]; initCounts.at(1)[1]+= si[k]; initCounts.at(1)[2]+= ii[k];
        }
        else {
            initCounts.at(2)[0]+= ss[k]; initCounts.at(2)[1]+= si[k]; initCounts.at(2)[2]+= ii[k];
        }
    }
    
    //Now we pass them to a 3x3 matrix of genotype frequencies per region (initFreq)
	vector<Genotype> initFreq;
    for (int r=0; r<3; ++r) {
		
        double initSS = max( 1.0, initCounts.at(r)[0] )/ (max( 1.0, initCounts.at(r)[0])+ max( 1.0, initCounts.at(r)[1]) + max( 1.0, initCounts.at(r)[2]));
		double initSI = max( 1.0, initCounts.at(r)[1] )/ (max( 1.0, initCounts.at(r)[0])+ max( 1.0, initCounts.at(r)[1]) + max( 1.0, initCounts.at(r)[2]));
		double initII = max( 1.0, initCounts.at(r)[2] )/ (max( 1.0, initCounts.at(r)[0])+ max( 1.0, initCounts.at(r)[1]) + max( 1.0, initCounts.at(r)[2]));
        
		Genotype dumGeno = {initSS,initSI, initII};
        initFreq.push_back(dumGeno);
		
	}
    initFreq.at(0) = deterministic(vec[0], vec[1]);
	initFreq.at(1) = deterministic(vec[2], vec[3]);
	initFreq.at(2) = deterministic(vec[4], vec[5]);
    
    
 	double lattice_h = southpoint / ((double) N_lpoints);
    int SH_limitpoint = ceil(borderSH / lattice_h);                 // gives the first lattice point past the change of region
    int HR_limitpoint = ceil(borderHR / lattice_h);
	
    
	// We will use this vector of populations in the simulation. We will fill it up with initial frequencies
    vector<Population> lat;
    
    for(int i=0; i<N_lpoints+1; ++i){
        if          (i< SH_limitpoint)  { Population newpop(initFreq[0]); lat.push_back(newpop);}
        else if     (i < HR_limitpoint) { Population newpop(initFreq[1]); lat.push_back(newpop);}
        else                            { Population newpop(initFreq[2]); lat.push_back(newpop);}
    }
   	
   	//create a normal distribution from which we will calculate the migration kernel values
	normal dispersion(0, sigma);
	
	//adjust the desired sampling of the migration kernel to the distances of the lattice
	double maxDistance= 3*sigma;
	int nPts_side=((double)N_migpoints-1.0)*0.5;
	int targetPoint = ceil(maxDistance / lattice_h);
	int adjustedPoint = targetPoint + (nPts_side - targetPoint%nPts_side);
	int step = adjustedPoint / nPts_side;
	
    vector<double> mig_prob;
	vector<int> mig_points;
	
  	//We start at the leftmost end of the migration kernel point
    double k= -(double)adjustedPoint*lattice_h ;
	int s = -adjustedPoint; 
    // Filling in the set of points that will be in the migration kernel
    
	for(int m=0; m< N_migpoints; ++m){
		double kernEval = k+(double)step*lattice_h*m;
        double prob = pdf(dispersion, kernEval);
		mig_prob.push_back(prob);
		int score = s+ step*m;
		mig_points.push_back(score);
        
	}
	
	// RUN!
	simulate(lat, mig_prob, mig_points, W, F, epsilon_one_run, maxGens_one_run, SH_limitpoint, HR_limitpoint, lattice_h);	
    
    vector<Genotype> interpol; 
	
	for (int i=0; i < nPops; ++i){
        int rightpoint = ceil(pop_positions.at(i) / lattice_h );   // lattice point to the right of the current pop 
        int leftpoint = max(1,rightpoint) -1;
        double gain = pop_positions.at(i) - (double)leftpoint*lattice_h;
        
        double SStemp = interpolate(lat.at(leftpoint).getAdults().SS, lat.at(rightpoint).getAdults().SS, gain, lattice_h);
        double SItemp = interpolate(lat.at(leftpoint).getAdults().SI, lat.at(rightpoint).getAdults().SI, gain, lattice_h);
        double IItemp = interpolate(lat.at(leftpoint).getAdults().II, lat.at(rightpoint).getAdults().II, gain, lattice_h);
        Genotype temp ={SStemp, SItemp, IItemp};
        if(SStemp+SItemp+IItemp >1.0) {
            Genotype fixtemp={SStemp/(SStemp+SItemp+IItemp), SItemp/(SStemp+SItemp+IItemp), IItemp/(SStemp+SItemp+IItemp)};
            temp=fixtemp;
        }  
        interpol.push_back(temp);
    }
    
	
    for (int k=0; k< nPops ; ++k ) {
		
        if(ss[k]!=0 && log(interpol.at(k).SS) != 0){ret += -ss[k]* max(log(interpol.at(k).SS), -penalty);}
 		if(si[k]!=0 && log(interpol.at(k).SI) != 0){ret += -si[k]* max(log(interpol.at(k).SI), -penalty);}
 		if(ii[k]!=0 && log(interpol.at(k).II) != 0){ret += -ii[k]* max(log(interpol.at(k).II), -penalty);}	
		
    }
	
    if(ret==NaN || ret!=ret ) { // according to IEEE standards, n!=n when n it is nan
		cout<<"WARNING->likelihood=NaN"<<endl;
		
		return penalty ;
	}
	return ret ;
	
	
	
}
Doub ctwo (VecDoub_I & vec ) {
	
    
	double epsilon_one_run=0.00000001;// typically 0.00000001
	int maxGens_one_run = 1000000;
	int N_migpoints=7;
	int N_lpoints=200;
    
	Doub  ret = 0;
	
	Doub penalty=1000000;
	
	// bounds for fitness
	for(int i=0;i<4;++i){
		if(vec[i]< 0)return penalty * (1.0+fabs(vec[i]));
		if(vec[i]>10000) return penalty * (1.0+fabs(vec[i]));
	}    
    
    // vec 0 thru 1 are the fitness parameters, go into the Fitness vector
    vector< Genotype > W;
	Genotype    dumW0 ={vec[0], 1, vec[1]}; W.push_back(dumW0);   
    Genotype    dumW1 ={vec[2], 1, vec[3]}; W.push_back(dumW1); 
    
    // vec4 is the assortative mating parameter, goes into F
    double F= vec[4];  
    // bounds for F
	if(F>1||F<0) return penalty * (1.0+fabs(F));
    
    //Assigning vec 5 to the migration kernel Standard deviation
    double sigma = vec[5];
    // bounds for migration distance
	if(sigma<0.5 || sigma > 50) return penalty * (1.0+fabs(sigma));
    
    //assignigin to division of original zones
    double borderZ= vec[6];
	//bounds for region Borders
	if(borderZ  <= 200 || borderZ >= 600) return penalty ;
    
    
    
    
    
    
    //************ DA hard coded, ugh, observed counts of mosquitoes per population
    // number of populations sampled
    int nPops = 105; 
    
	double ss [105] = {1, 3, 1, 3, 2, 4, 2, 8, 3, 2, 10, 12, 14, 2, 3, 5, 13, 1, 6, 13, 14, 19, 11, 6, 17, 4, 1, 0, 1, 4, 6, 10, 2, 3, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 1, 1, 0, 0, 0, 0, 7, 0, 0, 2, 1, 0, 0, 0, 0, 1, 0, 0, 0, 2, 0, 1, 0, 0, 0, 0, 1, 3, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0};
	double si [105] = {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 1, 1, 2, 0, 0, 0, 0, 0, 2, 0, 1, 4, 0, 1, 2, 3, 0, 4, 4, 0, 2, 0, 5, 11, 3, 2, 6, 0, 3, 9, 2, 2, 0, 1, 1, 1, 0, 3, 8, 5, 2, 0, 9, 0, 1, 4, 5, 1, 2, 2, 4, 2, 0, 1, 1, 1, 2, 0, 0, 0, 0, 0, 1, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0};
	double ii [105] = {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 2, 2, 10, 4, 9, 1, 9, 1, 4, 5, 10, 3, 10, 3, 5, 1, 3, 2, 7, 1, 1, 13, 1, 1, 8, 3, 0, 1, 2, 1, 0, 1, 0, 5, 11, 5, 2, 7, 1, 0, 2, 11, 5, 2, 1, 2, 2, 1, 4, 0, 1, 3, 9, 9, 3, 3, 12, 39, 6, 4, 22, 20, 1, 4, 16, 3, 3, 1, 5, 1, 3, 5, 17, 6, 4, 3, 3, 2};
    
	// We use these, in Km from the first population North. POPS ARE NOT IN NORTH-SOUTH order, but it doesn't matter. 
	double latitude[105]={0,2.938,10.675,18.507,52.221,63.901,69.25,85.587,81.441,82.168,90.438,91.633,109.558,97.175,111.404,119.68,121.147,128.242,133.919,138.991,144.977,150.566,154.861,159.061,166.785,170.196,177.031,180.449,186.593,190.01,198.75,205.613,219.635,220.634,279.076,289.466,293.722,298.591,301.465,302.638,308.192,313.6,340.467,345.168,350.155,360.459,360.459,370.288,377.258,398.46,410.57,433.066,432.313,432.677,436.069,436.443,443.577,448.002,442.978,438.65,433.854,432.867,424.969,421.92,419.647,427.694,434.556,463.059,476.454,479.925,493.065,494.054,503.314,512.845,516.067,517.747,518.373,522.384,556.835,626.619,637.819,638.387,644.188,647.574,649.78,681.256,689.085,693.152,694.767,699.64,702.322,708.546,716.069,725.535,740.999,744.774,759.76,754.36,822.137,839.398,840.421,853.288,851.983,852.945,852.049};
    
	//passing it to a vector, finding the southmost point
    vector<double> pop_positions; 
	double southpoint=0;
    for (int k=0; k < nPops; ++k ) { pop_positions.push_back(latitude[k]); if (southpoint < latitude[k]) southpoint = latitude[k];}
    
    //Now we pass them to a 3x3 matrix of genotype frequencies per region (initFreq)
	vector<Genotype> initFreq;
    
    initFreq.resize(2);
    initFreq.at(0) = deterministic(vec[0], vec[1]);
	initFreq.at(1) = deterministic(vec[2], vec[3]);
	// initFreq.at(2) = deterministic(vec[4], vec[5]);
    
    
	double lattice_h = southpoint / ((double) N_lpoints);
    int Z_limitpoint = ceil(borderZ / lattice_h);                 // gives the first lattice point past the change of region
    
    
	// We will use this vector of populations in the simulation. We will fill it up with initial frequencies
    vector<Population> lat;
    
    for(int i=0; i<N_lpoints+1; ++i){
        if          (i< Z_limitpoint)  { Population newpop(initFreq[0]); lat.push_back(newpop);}
        else                            { Population newpop(initFreq[1]); lat.push_back(newpop);}
    }
   	
   	//create a normal distribution from which we will calculate the migration kernel values
	normal dispersion(0, sigma);
	
	//adjust the desired sampling of the migration kernel to the distances of the lattice
	double maxDistance= 3*sigma;
	int nPts_side=((double)N_migpoints-1.0)*0.5;
	int targetPoint = ceil(maxDistance / lattice_h);
	int adjustedPoint = targetPoint + (nPts_side - targetPoint%nPts_side);
	int step = adjustedPoint / nPts_side;
	
    vector<double> mig_prob;
	vector<int> mig_points;
	
  	//We start at the leftmost end of the migration kernel point
    double k= -(double)adjustedPoint*lattice_h ;
	int s = -adjustedPoint; 
    // Filling in the set of points that will be in the migration kernel
    
	for(int m=0; m< N_migpoints; ++m){
		double kernEval = k+(double)step*lattice_h*m;
        double prob = pdf(dispersion, kernEval);
		mig_prob.push_back(prob);
		int score = s+ step*m;
		mig_points.push_back(score);
        
	}
	
    // this is a very high value for the borders. It allows us to use the same constructor as for three regions
    int dummyLimit = 100000;
	// RUN!
	simulate(lat, mig_prob, mig_points, W, F, epsilon_one_run, maxGens_one_run, Z_limitpoint, dummyLimit, lattice_h);	
    
    vector<Genotype> interpol; 
	
    for (int i=0; i < nPops; ++i){
        int rightpoint = ceil(pop_positions.at(i) / lattice_h );   // lattice point to the right of the current pop 
        int leftpoint = max(1,rightpoint) -1;
        double gain = pop_positions.at(i) - (double)leftpoint*lattice_h;
        
        double SStemp = interpolate(lat.at(leftpoint).getAdults().SS, lat.at(rightpoint).getAdults().SS, gain, lattice_h);
        double SItemp = interpolate(lat.at(leftpoint).getAdults().SI, lat.at(rightpoint).getAdults().SI, gain, lattice_h);
        double IItemp = interpolate(lat.at(leftpoint).getAdults().II, lat.at(rightpoint).getAdults().II, gain, lattice_h);
        Genotype temp ={SStemp, SItemp, IItemp};
        if(SStemp+SItemp+IItemp >1.0) {
            Genotype fixtemp={SStemp/(SStemp+SItemp+IItemp), SItemp/(SStemp+SItemp+IItemp), IItemp/(SStemp+SItemp+IItemp)};
            temp=fixtemp;
        }  
        interpol.push_back(temp);
    }
    
	
    for (int k=0; k< nPops ; ++k ) {
        
        if(ss[k]!=0 && log(interpol.at(k).SS) != 0){ret += -ss[k]* max(log(interpol.at(k).SS), -penalty);}
 		if(si[k]!=0 && log(interpol.at(k).SI) != 0){ret += -si[k]* max(log(interpol.at(k).SI), -penalty);}
 		if(ii[k]!=0 && log(interpol.at(k).II) != 0){ret += -ii[k]* max(log(interpol.at(k).II), -penalty);}	
        
    }
	
    if(ret==NaN || ret!=ret ) { // according to IEEE standards, n!=n when n it is nan
		cout<<"WARNING->likelihood=NaN"<<endl;
        return penalty ;
	}
	return ret ;
	
	
	
}
Doub cone (VecDoub_I & vec ) {
	
    
	double epsilon_one_run=0.00000001;// typically 0.00000001
	int maxGens_one_run = 100000;
	int N_migpoints=7;
	int N_lpoints=200;
    
	Doub  ret = 0;
	
	Doub penalty=1000000;
	
	// bounds for fitness
	for(int i=0;i<2;++i){
		if(vec[i]< 0)return penalty * (1.0+fabs(vec[i]));
		if(vec[i]>10000) return penalty * (1.0+fabs(vec[i]));
	}
	
    // bounds for F
	if(vec[2]>1||vec[2]<0) return penalty * (1.0+fabs(vec[2]));
    
    // vec2 is the assortative mating parameter, goes into F
    double F= vec[2];  
	
    // bounds for migration distance
	if(vec[3]<0.5 || vec[3] > 20) return penalty * (1.0+fabs(vec[3]));
    //Assigning vec 7 to the migration kernel Standard deviation
    double sigma = vec[3];
    
    //assignigin to division of original zones
    double borderZ= vec[4];
    
	//bounds for region Borders
	if(borderZ  <= 300 || borderZ >= 600) return penalty ;
    
    // vec 0 thru 1 are the fitness parameters, go into the Fitness vector
    vector< Genotype > W;
	Genotype    dumW0 ={vec[0], 1, vec[1]}; W.push_back(dumW0);   
    // Genotype    dumW1 ={vec[2], 1, vec[3]}; W.push_back(dumW1); 
    // Genotype    dumW2 ={vec[4], 1, vec[5]}; W.push_back(dumW2); 
    
    
    
    
    
    //************ DA hard coded, ugh, observed counts of mosquitoes per population
    // number of populations sampled
    int nPops = 105; 
    
	double ss [105] = {1, 3, 1, 3, 2, 4, 2, 8, 3, 2, 10, 12, 14, 2, 3, 5, 13, 1, 6, 13, 14, 19, 11, 6, 17, 4, 1, 0, 1, 4, 6, 10, 2, 3, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 1, 1, 0, 0, 0, 0, 7, 0, 0, 2, 1, 0, 0, 0, 0, 1, 0, 0, 0, 2, 0, 1, 0, 0, 0, 0, 1, 3, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0};
	double si [105] = {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 1, 1, 2, 0, 0, 0, 0, 0, 2, 0, 1, 4, 0, 1, 2, 3, 0, 4, 4, 0, 2, 0, 5, 11, 3, 2, 6, 0, 3, 9, 2, 2, 0, 1, 1, 1, 0, 3, 8, 5, 2, 0, 9, 0, 1, 4, 5, 1, 2, 2, 4, 2, 0, 1, 1, 1, 2, 0, 0, 0, 0, 0, 1, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0};
	double ii [105] = {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 2, 2, 10, 4, 9, 1, 9, 1, 4, 5, 10, 3, 10, 3, 5, 1, 3, 2, 7, 1, 1, 13, 1, 1, 8, 3, 0, 1, 2, 1, 0, 1, 0, 5, 11, 5, 2, 7, 1, 0, 2, 11, 5, 2, 1, 2, 2, 1, 4, 0, 1, 3, 9, 9, 3, 3, 12, 39, 6, 4, 22, 20, 1, 4, 16, 3, 3, 1, 5, 1, 3, 5, 17, 6, 4, 3, 3, 2};
    
	// We use these, in Km from the first population North. POPS ARE NOT IN NORTH-SOUTH order, but it doesn't matter. 
	double latitude[105]={0,2.938,10.675,18.507,52.221,63.901,69.25,85.587,81.441,82.168,90.438,91.633,109.558,97.175,111.404,119.68,121.147,128.242,133.919,138.991,144.977,150.566,154.861,159.061,166.785,170.196,177.031,180.449,186.593,190.01,198.75,205.613,219.635,220.634,279.076,289.466,293.722,298.591,301.465,302.638,308.192,313.6,340.467,345.168,350.155,360.459,360.459,370.288,377.258,398.46,410.57,433.066,432.313,432.677,436.069,436.443,443.577,448.002,442.978,438.65,433.854,432.867,424.969,421.92,419.647,427.694,434.556,463.059,476.454,479.925,493.065,494.054,503.314,512.845,516.067,517.747,518.373,522.384,556.835,626.619,637.819,638.387,644.188,647.574,649.78,681.256,689.085,693.152,694.767,699.64,702.322,708.546,716.069,725.535,740.999,744.774,759.76,754.36,822.137,839.398,840.421,853.288,851.983,852.945,852.049};
    
	//passing it to a vector, finding the southmost point
    vector<double> pop_positions; 
	double southpoint=0;
    for (int k=0; k < nPops; ++k ) { pop_positions.push_back(latitude[k]); if (southpoint < latitude[k]) southpoint = latitude[k];}
    
    
    // This one is to get the average frequencies in a region. First, we get the counts per region (initCounts)
    /*
     vector< vector<double> > initCounts;
     vector<double> zeroDum (3,0) ; for (int i =0; i<3; ++i) initCounts.push_back(zeroDum);
     
     for (int k=0; k < nPops; ++k ) {          
     if (pop_positions.at(k) < borderZH) {
     initCounts.at(0)[0]+= ss[k]; initCounts.at(0)[1]+= si[k]; initCounts.at(0)[2]+= ii[k];
     }
     else if (pop_positions.at(k) < borderHR){
     initCounts.at(1)[0]+= ss[k]; initCounts.at(1)[1]+= si[k]; initCounts.at(1)[2]+= ii[k];
     }
     else {
     initCounts.at(2)[0]+= ss[k]; initCounts.at(2)[1]+= si[k]; initCounts.at(2)[2]+= ii[k];
     }
     }
     */  
    //Now we pass them to a 3x3 matrix of genotype frequencies per region (initFreq)
	vector<Genotype> initFreq;
    
    /*
     for (int r=0; r<3; ++r) {
     
     double initSS = max( 1.0, initCounts.at(r)[0] )/ (max( 1.0, initCounts.at(r)[0])+ max( 1.0, initCounts.at(r)[1]) + max( 1.0, initCounts.at(r)[2]));
     double initSI = max( 1.0, initCounts.at(r)[1] )/ (max( 1.0, initCounts.at(r)[0])+ max( 1.0, initCounts.at(r)[1]) + max( 1.0, initCounts.at(r)[2]));
     double initII = max( 1.0, initCounts.at(r)[2] )/ (max( 1.0, initCounts.at(r)[0])+ max( 1.0, initCounts.at(r)[1]) + max( 1.0, initCounts.at(r)[2]));
     
     Genotype dumGeno = {initSS,initSI, initII};
     initFreq.push_back(dumGeno);
     
     }
     */
    
    initFreq.resize(2);
    Genotype fixedS ={1.0,0,0};
    Genotype fixedI ={0,0,1.0};
    
    initFreq.at(0) = fixedS;
	initFreq.at(1) = fixedI;
	// initFreq.at(2) = deterministic(vec[4], vec[5]);
    
    
	double lattice_h = southpoint / ((double) N_lpoints);
    int Z_limitpoint = ceil(borderZ / lattice_h);                 // gives the first lattice point past the change of region
    
    //int HR_limitpoint = ceil(borderHR / lattice_h);
	
    
	// We will use this vector of populations in the simulation. We will fill it up with initial frequencies
    vector<Population> lat;
    
    for(int i=0; i<N_lpoints+1; ++i){
        if          (i< Z_limitpoint)  { Population newpop(initFreq[0]); lat.push_back(newpop);}
        //else if     (i < HR_limitpoint) { Population newpop(initFreq[1]); lat.push_back(newpop);}
        else                            { Population newpop(initFreq[1]); lat.push_back(newpop);}
    }
   	
   	//create a normal distribution from which we will calculate the migration kernel values
	normal dispersion(0, sigma);
	
	//adjust the desired sampling of the migration kernel to the distances of the lattice
	double maxDistance= 3*sigma;
	int nPts_side=((double)N_migpoints-1.0)*0.5;
	int targetPoint = ceil(maxDistance / lattice_h);
	int adjustedPoint = targetPoint + (nPts_side - targetPoint%nPts_side);
	int step = adjustedPoint / nPts_side;
	
    vector<double> mig_prob;
	vector<int> mig_points;
	
  	//We start at the leftmost end of the migration kernel point
    double k= -(double)adjustedPoint*lattice_h ;
	int s = -adjustedPoint; 
    // Filling in the set of points that will be in the migration kernel
    
	for(int m=0; m< N_migpoints; ++m){
		double kernEval = k+(double)step*lattice_h*m;
        double prob = pdf(dispersion, kernEval);
		mig_prob.push_back(prob);
		int score = s+ step*m;
		mig_points.push_back(score);
        
	}
	
    // this is a very high value for the borders, that causes all points in the lattice to be in region 0. I don't want to change the code of the sim
    int dummyLimit = 100000;
	// RUN!
	simulate(lat, mig_prob, mig_points, W, F, epsilon_one_run, maxGens_one_run, dummyLimit, dummyLimit, lattice_h);	
    
    vector<Genotype> interpol; 
	
    for (int i=0; i < nPops; ++i){
        int rightpoint = ceil(pop_positions.at(i) / lattice_h );   // lattice point to the right of the current pop 
        int leftpoint = max(1,rightpoint) -1;
        double gain = pop_positions.at(i) - (double)leftpoint*lattice_h;
        
        double SStemp = interpolate(lat.at(leftpoint).getAdults().SS, lat.at(rightpoint).getAdults().SS, gain, lattice_h);
        double SItemp = interpolate(lat.at(leftpoint).getAdults().SI, lat.at(rightpoint).getAdults().SI, gain, lattice_h);
        double IItemp = interpolate(lat.at(leftpoint).getAdults().II, lat.at(rightpoint).getAdults().II, gain, lattice_h);
        Genotype temp ={SStemp, SItemp, IItemp};
        if(SStemp+SItemp+IItemp >1.0) {
            Genotype fixtemp={SStemp/(SStemp+SItemp+IItemp), SItemp/(SStemp+SItemp+IItemp), IItemp/(SStemp+SItemp+IItemp)};
            temp=fixtemp;
        }  
        interpol.push_back(temp);
    }
    
	
    for (int k=0; k< nPops ; ++k ) {
        
        if(ss[k]!=0 && log(interpol.at(k).SS) != 0){ret += -ss[k]* max(log(interpol.at(k).SS), -penalty);}
 		if(si[k]!=0 && log(interpol.at(k).SI) != 0){ret += -si[k]* max(log(interpol.at(k).SI), -penalty);}
 		if(ii[k]!=0 && log(interpol.at(k).II) != 0){ret += -ii[k]* max(log(interpol.at(k).II), -penalty);}	
        
    }
	
    if(ret==NaN || ret!=ret ) { // according to IEEE standards, n!=n when n it is nan
		cout<<"WARNING->likelihood=NaN"<<endl;
        
        /*  for (int k=0; k< nPops ; ++k ) {
         if(ss[k]!=0){cout<<ss[k]<<" "<<max(log(interpol.at(k).SS), -penalty)<<" -- "; }
         if(si[k]!=0){cout<<si[k]<<" "<<max(log(interpol.at(k).SI), -penalty)<<" -- "; }
         if(ii[k]!=0){cout<<ii[k]<<" "<<max(log(interpol.at(k).II), -penalty)<<" -- "; }	
         }        
         cout<<ret<<endl;
         */
		return penalty ;
	}
	return ret ;
	
	
	
}

