//
//  main.cpp
//  inv2lafit
//   
/*This is a wrapper to run one of 10 models to fit the data on
Ayala et al, "Reproductive Isolation and Local Adaptation Quantified" (Accepted in Evolution, Oct 2, 2012)

 
There is a lot of repetition on the functions for historical reasons.
We were committed to using the Numerical Recipes algorithm for Simulated Annealing, which happened to be not friendly  with other code 
Most importantly, we will share only our code, but this will not compile without the header files from Numerical Recipes
As partially responsible for this code, I apologize for the ugliness of it.
You can contact me with any questions or comments: r.guerrero@utexas.edu

 The model that will me run needs to be defined in preprocessing 
 For example, this file will be shared such that it would build SS_3 (stepping stone model with three regions)
 All defines is in the common_snippets.h header file
 
 The models are:
 SS_1 , SS_2 , SS_3  ::: Stepping stone one, two and three regions, respectively
 C_1 , C_2, C_3 ::: Continuous space with one, two and three regions
 SS_3_F0 , C_3_F0 ::: Models with F = 0, leading to all random mating
 SS_3_FGeno and SS_3_FRegion ::: models in which the level of assortment varies by Genotype and Region respectively.
*/
//  Copyright (c) 2012 University of Texas. All rights reserved.
//

#include <iostream>

#include "amebsa.h" //From NumRecipes, not included in shared package
#include "nr3.h"//From NumRecipes, not included in shared package
#include "ran.h"//From NumRecipes, not included in shared package

#include "models.h"

#include "common_snippets.h"

int main(int argc, const char * argv[])
{
 
  	stringstream general;
	if(argc > 1 ) general <<"startvec"<<argv[1]<<".txt"; 
    ifstream fin( general.str().c_str());
	if(!fin.is_open()){cerr<<
        "Where's your startvec file?\nRemember to specify the Model Number in args\n"
        <<"SS_1 = 1\nSS_2 = 2\nSS_3 = 3, needs borders.txt\nSS_3_F0 = 4, needs borders.txt\nSS_3_FR = 5, needs borders.txt\nSS_3_FG = 6, needs borders.txt\n"
        <<"C_1 = 7\nC_2 = 8\nC_3 = 9\nC_3_F0 = 10\n";return 1;}
	string line;
	istringstream iss(line);
	
	double doubtemp;
	vector < vector<double> > sets;

	int nSets=0;
	while (!fin.eof()){
		++nSets;
		iss.clear();
		vector<double> oneset;
		getline( fin, line);
		iss.str(line);
		while(iss >> doubtemp){
			oneset.push_back(doubtemp);
		}
		sets.push_back(oneset);
		
	}
	
    int nPars;
    nPars=sets.at(0).size();
    
    VecDoub vec(nPars);
    VecDoub dels(nPars);
    for(int i =0; i< vec.size(); ++i){vec[i] = sets.at(0)[i];}
    for(int i =0; i< dels.size(); ++i){dels[i] = sets.at(1)[i];}

	if(nSets==0){ cerr<< "Empty input file. No chains to be run? Exiting (1)\n"; return 1;}
    if(nSets==1){ cerr<< "Forgot to include the deltas in the second row of input file. Exiting (1)\n"; return 1;}
    
    
        
	int iter;				// number of iterations
	Doub ftol;              // starting tolerance normally = 0.0001
	Doub temperature;		//starting temp
	Doub tmp_step;	
	
    SA_INITIALIZE 
    
    Doub (*funcpointer)(VecDoub_I & vec );
    stringstream ss;
    
    int modelN = 0;
    
    if(argc>1) sscanf (argv[1],"%i", &modelN);
    
    WHICH_MODEL(modelN) 
         
	ofstream fout( ss.str().c_str() );  // output file stream

    Amebsa< Doub(VecDoub_I &)> amebsa(vec, dels, *funcpointer, ftol);

    int it =iter;
	Bool converged = amebsa.anneal (it, temperature); 
    
	while(!converged ){
		
		temperature= temperature*tmp_step;
		it=iter;
		converged=amebsa.anneal (it, temperature);
    }
	
    
	MatDoub vec_chk=amebsa.p;
	Doub ml=amebsa.yb;
	
	int j=0;
	while(j<3){
		
		Amebsa< Doub(VecDoub_I &)> Secondrun(vec_chk, *funcpointer, 0.1*ftol);
        
		
		temperature=1;//pow(1.0,j);
		Doub chk_temp_step=0.75;
		it =iter;
		Bool check = Secondrun.anneal (it, temperature); 
		
		while(!check ){
			
			temperature= temperature*chk_temp_step;
			it =iter;
			check=Secondrun.anneal (it, temperature);
		}
		ml=Secondrun.yb;
		vec_chk=Secondrun.p;
		++j;
        
        
        cout<<"Temporary Max::\n";
        for (int j=0; j< nPars; ++j){
			cout<<Secondrun.pb[j]<<" ";
		}
        cout<<ml<<endl;
        
	}
	
	Amebsa< Doub(VecDoub_I &)> thirdrun(vec_chk, *funcpointer, ftol);
    
	temperature=1;//pow(1.0,j);
	Doub chk_temp_step=0.95;
	it =iter;
	Bool check = thirdrun.anneal (it, temperature); 
    
    while(!check ){
        
        temperature= temperature*chk_temp_step;
        it =iter;
        check=thirdrun.anneal (it, temperature);
    }
    ml=thirdrun.yb;
    vec_chk=thirdrun.p;
    
    
    int m=0;
    while(m<5){
        
        Amebsa< Doub(VecDoub_I &)> third_chk(vec_chk, *funcpointer, 0.1*ftol);
        
        
        temperature=0.5;
        Doub chk_temp_step=0.9;
        it =iter;
        Bool check = third_chk.anneal (it, temperature); 
        
        while(!check ){
            temperature= temperature*chk_temp_step;
            it =iter;
            check=third_chk.anneal (it, temperature);
            
        }
        ml=third_chk.yb;
        vec_chk=third_chk.p;
        ++m;
        
        cout<<"Refining has found:\n";
        for (int j=0; j< nPars; ++j){
            cout<<third_chk.pb[j]<<" ";
        }
        cout<<ml<<endl;
        
    }
    
    
	fout.precision(8);
	for (int j=0; j< nPars; ++j){
			fout<<vec_chk[j]<<" ";
		}
	fout<<ml<<endl;
	fout.close();
    
    
	return 0;
}
