/*
 *  population.cpp
 *  Cameroon
 *
 *  Created by Rafael Guerrero on 3/30/09.
 *  Copyright 2009 . All rights reserved.
 *
 */

#include<math.h>
#include<iostream>
using std::cout;
#include <algorithm>
using std::max;

#include<vector>
using std::vector;

#include "population.h"

std::ostream& operator<<(std::ostream& os, const Genotype& g){ os <<"SS: " << g.SS <<", SI: "<<g.SI <<", II: "<< g.II; return os;}


Population::Population (const Genotype & initfreq ) : larvae(initfreq) {
}

Population::~Population(){
	
}

void Population::goVirgins(const Genotype & W ){
	
	//MeanW = W.SS * larvae.SS + W.SI * larvae.SI + W.II * larvae.II;
	double temp=0.0;
	temp= W.SS * larvae.SS ;
	temp+= W.SI * larvae.SI ;
	temp+= W.II * larvae.II ;
	virgins.SS= W.SS * larvae.SS /temp;
	virgins.SI= W.SI * larvae.SI /temp;
	virgins.II= W.II * larvae.II /temp;
}

void Population::goZygotes(const double& F){
	
	
	zygotes.SS= F * (virgins.SS + virgins.SI / 4) + (1- F) * pow( virgins.SS + virgins.SI / 2 ,2);
	zygotes.SI= F * virgins.SI / 2 + 2 * (1-F) * (virgins.SS + virgins.SI / 2) * (virgins.SI / 2 + virgins.II) ;
	zygotes.II= F * (virgins.II + virgins.SI / 4) + (1- F) * pow( virgins.II + virgins.SI / 2 ,2);
	
}
/* for fertility selection:
 
 void Population::goVirgins(const Genotype & W ){
 
 //MeanW = W.SS * larvae.SS + W.SI * larvae.SI + W.II * larvae.II;
 double temp=0.0;
 temp= W.SS * larvae.SS ;
 temp+= W.SI * larvae.SI ;
 temp+= W.II * larvae.II ;
 virgins.SS=  larvae.SS;
 virgins.SI=  larvae.SI;
 virgins.II=  larvae.II ;
 }
 
 void Population::goZygotes(const double& F, const Genotype & W){
 
 
 zygotes.SS= F * (virgins.SS + virgins.SI / 4) + (1- F) * pow( virgins.SS + virgins.SI / 2 ,2);
 zygotes.SI= F * virgins.SI / 2 + 2 * (1-F) * (virgins.SS + virgins.SI / 2) * (virgins.SI / 2 + virgins.II) ;
 zygotes.II= F * (virgins.II + virgins.SI / 4) + (1- F) * pow( virgins.II + virgins.SI / 2 ,2);
 
 double temp=0.0;
 temp= W.SS * zygotes.SS ;
 temp+= W.SI * zygotes.SI ;
 temp+= W.II * zygotes.II ;
 zygotes.SS= W.SS * zygotes.SS /temp;
 zygotes.SI= W.SI * zygotes.SI /temp;
 zygotes.II= W.II * zygotes.II /temp;
 
 
 }
 */
void Population::goZygotes(const  vector<double> & F){
	//double tot =virgins.SS + virgins.SI + virgins.II;
	//cout<<"in "<< tot <<'\n';
	
	Genotype pool;
	pool.SS= (1- F[0])*virgins.SS;
	pool.SI= (1- F[1])*virgins.SI;
	pool.II= (1- F[2])*virgins.II;
	double poolT =pool.SS+ pool.SI + pool.II;
	pool.SS= pool.SS/poolT;pool.SI= pool.SI/poolT;pool.II= pool.II/poolT;
	//cout<<"pool "<<poolT<<'\n';
    
	Genotype poolOffs;
	poolOffs.SS=  pow( pool.SS + pool.SI / 2 ,2);
	poolOffs.SI= 2 * (pool.SS + pool.SI / 2) * (pool.SI / 2 + pool.II);
	poolOffs.II=  pow( pool.II + pool.SI / 2 ,2);
	 
	Genotype ass;
	ass.SS= F[0]*virgins.SS+ F[1]*virgins.SI/4;
	ass.SI= F[1]*virgins.SI/2;
	ass.II= F[2]*virgins.II+ F[1]*virgins.SI/4;
	
    
	zygotes.SS= ass.SS + poolT* poolOffs.SS ;
	zygotes.SI= ass.SI + poolT* poolOffs.SI ;
	zygotes.II= ass.II + poolT* poolOffs.II ;
    
	
	double out =zygotes.SS+ zygotes.SI + zygotes.II;
	if(fabs(1-out)>0.0000001)cout<<"zygotes off ="<<out<<'\n';
}
vector<double> getSimpson(int length){ 
    // These coefficients correspond to the extended Simpsons Rule for integration (eg. equation 4.1.13 in Numerical recipes)
    vector<double> out;
    for (int i=0; i< length;  ++i){
        if(i==0 || i==length-1) { out.push_back(1.0/3.0);}
        else if (i%2)/* is odd */ { out.push_back(4.0/3.0);}
        else { out.push_back(2.0/3.0);}
    }
    return out;
}


int itr(int ind, int high){ // this function fixes the index of a point in the lattice such that it becomes reflective 
    if (ind >= 0 && ind <=high) return ind;
    if (ind < 0 ) return  (-1*ind);
    if (ind > high ) return high - (ind - high );
    cout<<"Error in itr\n";
    return -1;
}

vector<int> createIndx(int pop, vector<int>& points,  int high){
    
    // if(DEBUG) cout<<"creating Index: ";
    vector<int> out;
    for(int i=0; i< points.size(); ++i){
        out.push_back(itr(pop + points[i], high)); // push back the absolute index -- reflective
        // if(DEBUG) cout<< itr(pop + points[i], high) <<" ";
    }
    //if(DEBUG) cout<<'\n';
    return out;
}

Genotype migKernel(vector<Population>& pop, const vector< double >& mig, double lattice_h, vector<int> indx, bool isLarvae ){
    
    Genotype temp={0,0,0};
    
    vector<double> SimpsonCoef = getSimpson(mig.size());    // get the list of coefficients for simpson integration
    
    for(int i=0; i < indx.size();++i){
        Genotype migrants = pop.at(indx[i]).virgins;
        if(isLarvae) migrants = pop.at(indx[i]).zygotes;
        
        temp.SS += lattice_h* migrants.SS * mig[i] * SimpsonCoef[i];
        temp.SI += lattice_h* migrants.SI * mig[i] * SimpsonCoef[i];
        temp.II += lattice_h* migrants.II * mig[i] * SimpsonCoef[i];
    }
    double total= temp.SS+temp.SI+temp.II;
    Genotype out = {temp.SS/total, temp.SI/total, temp.II/total};
    return out;
};



bool migration(vector<Population>& pop,  const vector< double >& mig, double epsilon){
	vector < Population > temp = pop;
	
	for (int j =0; j< pop.size(); ++j) {
        if (j==0) {
            pop[j].adults.SS= (1-mig[j]) * pop[j].virgins.SS + mig[j] * pop[j+1].virgins.SS;
            pop[j].adults.SI= (1-mig[j]) * pop[j].virgins.SI + mig[j] * pop[j+1].virgins.SI;
            pop[j].adults.II= (1-mig[j]) * pop[j].virgins.II + mig[j] * pop[j+1].virgins.II;
            
            pop[j].larvae.SS= (1-mig[j]) * pop[j].zygotes.SS + mig[j] * pop[j+1].zygotes.SS;
            pop[j].larvae.SI= (1-mig[j]) * pop[j].zygotes.SI + mig[j] * pop[j+1].zygotes.SI;
            pop[j].larvae.II= (1-mig[j]) * pop[j].zygotes.II + mig[j] * pop[j+1].zygotes.II;
        }
        
        else if (j==pop.size()-1) {
            pop[j].adults.SS= (1-mig[j-1]) * pop[j].virgins.SS + mig[j-1] * pop[j-1].virgins.SS;
            pop[j].adults.SI= (1-mig[j-1]) * pop[j].virgins.SI + mig[j-1] * pop[j-1].virgins.SI;
            pop[j].adults.II= (1-mig[j-1]) * pop[j].virgins.II + mig[j-1] * pop[j-1].virgins.II;
            
            pop[j].larvae.SS= (1-mig[j-1]) * pop[j].zygotes.SS + mig[j-1] * pop[j-1].zygotes.SS;
            pop[j].larvae.SI= (1-mig[j-1]) * pop[j].zygotes.SI + mig[j-1] * pop[j-1].zygotes.SI;
            pop[j].larvae.II= (1-mig[j-1]) * pop[j].zygotes.II + mig[j-1] * pop[j-1].zygotes.II;
        }
        else {
            pop[j].adults.SS= (1-mig[j-1]-mig[j]) * pop[j].virgins.SS + mig[j] * pop[j+1].virgins.SS + mig[j-1] * pop[j-1].virgins.SS;
            pop[j].adults.SI= (1-mig[j-1]-mig[j]) * pop[j].virgins.SI + mig[j] * pop[j+1].virgins.SI + mig[j-1] * pop[j-1].virgins.SI;
            pop[j].adults.II= (1-mig[j-1]-mig[j]) * pop[j].virgins.II + mig[j] * pop[j+1].virgins.II + mig[j-1] * pop[j-1].virgins.II;
            
            pop[j].larvae.SS= (1-mig[j-1]-mig[j]) * pop[j].zygotes.SS + mig[j] * pop[j+1].zygotes.SS + mig[j-1] * pop[j-1].zygotes.SS;
            pop[j].larvae.SI= (1-mig[j-1]-mig[j]) * pop[j].zygotes.SI + mig[j] * pop[j+1].zygotes.SI + mig[j-1] * pop[j-1].zygotes.SI;
            pop[j].larvae.II= (1-mig[j-1]-mig[j]) * pop[j].zygotes.II + mig[j] * pop[j+1].zygotes.II + mig[j-1] * pop[j-1].zygotes.II; 
            
        }
    }
	bool e=true;
    
	double delta_max=0;
	for (int j =0; j< pop.size(); ++j) {
		delta_max= max( fabs(pop.at(j).larvae.SS - temp.at(j).larvae.SS), delta_max);
		delta_max= max( fabs(pop.at(j).larvae.SI - temp.at(j).larvae.SI), delta_max);
		delta_max= max( fabs(pop.at(j).larvae.II - temp.at(j).larvae.II), delta_max);
	}
	if(delta_max < epsilon) e=false; 
	
	return e;
}

bool migration(vector<Population>& pop,  const vector< double >& mig, vector<int>& mig_points, double lattice_h, double epsilon){
	vector < Population > temp = pop;
	
    
    for (int j =0; j< pop.size(); ++j) {
        vector<int> indx = createIndx(j, mig_points,pop.size()-1); // get the list absolute positions that will contribute to migration
        
        pop[j].adults = migKernel(pop, mig, lattice_h, indx, false);
        pop[j].larvae = migKernel(pop, mig, lattice_h, indx, true);
    }
    
	bool e=true;
    
	double delta_max=0;
	for (int j =0; j< pop.size(); ++j) {
		delta_max= max( fabs(pop.at(j).larvae.SS - temp.at(j).larvae.SS), delta_max);
		delta_max= max( fabs(pop.at(j).larvae.SI - temp.at(j).larvae.SI), delta_max);
		delta_max= max( fabs(pop.at(j).larvae.II - temp.at(j).larvae.II), delta_max);
	}
	if(delta_max < epsilon) e=false; 
	
	return e;
}

Genotype Population::getVirgins() {
	return virgins;
}
Genotype Population::getZygotes() {
	return zygotes;
}
Genotype Population::getAdults() {
	return adults;
}
Genotype Population::getLarvae() {
	return larvae;
}

int detectRegion(int current, int border1, int border2){
    if(current >= border2) return 2;
    if(current >= border1) return 1;
    return 0;
}

void simulate(vector<Population>& pop, 
              const vector< double >& mig,
              vector<int>& mig_points,
              const vector<Genotype> & W, 
              const double & F, 
              double & epsilon, 
              int & maxGens,
              int & s_h,
              int & h_r, 
              double lattice_h  ) {
    
	bool end=true;
	int generation=0;
	while (end && generation< maxGens){
		for (int i=0; i<pop.size(); ++i){
			int region = detectRegion(i,s_h,h_r);
            pop[i].goVirgins(W[region]);
			pop[i].goZygotes(F);
		}
        
        end=migration(pop, mig, mig_points, lattice_h, epsilon);
        
		++generation; 
		if (generation == maxGens) cout<<"max number of generations exceeded\n"<<W.at(0).SI<<" "<<W.at(0).II<<" "<<W.at(1).SI<<" "<<W.at(1).II<<" "<<W.at(2).SI<<" "<<W.at(2).II<<'\n';
        
	}
	
}

vector<Genotype> simulate(vector<Population> pop, 
                          const vector< double >& mig, 
                          const vector<Genotype> & W, 
                          const double & F, 
                          double epsilon=0.000001, 
                          int maxGens=100000) {
    
	bool end=true;
	int generation=0;
	while (end && generation< maxGens){
		for (int i=0; i<pop.size(); ++i){
			pop[i].goVirgins(W[i]);
			pop[i].goZygotes(F);
		}
        
        end=migration(pop, mig, epsilon);
        
		++generation; 
		if (generation == maxGens) cout<<"max number of generations exceeded\n"<<W.at(0).SI<<" "<<W.at(0).II<<" "<<W.at(1).SI<<" "<<W.at(1).II<<" "<<W.at(2).SI<<" "<<W.at(2).II<<'\n';
        
	}
	vector<Genotype> output;
	for (int j =0; j< pop.size(); ++j) {
		output.push_back(pop[j].getAdults());
	}
	
	return output;
	
}

vector<Genotype> simulate(vector<Population> pop, 
                          const vector< double >& mig, 
                          const vector<Genotype> & W, 
                          const vector< double >& F, 
                          double epsilon=0.000001, 
                          int maxGens=100000) {
    
	bool end=true;
	int generation=0;
	while (end && generation< maxGens){
		for (int i=0; i<pop.size(); ++i){
			pop[i].goVirgins(W[i]);
			pop[i].goZygotes(F[i]);
		}
        
        end=migration(pop, mig, epsilon);
        
		++generation; 
		if (generation == maxGens) cout<<"max number of generations exceeded\n"<<W.at(0).SI<<" "<<W.at(0).II<<" "<<W.at(1).SI<<" "<<W.at(1).II<<" "<<W.at(2).SI<<" "<<W.at(2).II<<'\n';
        
	}
	vector<Genotype> output;
	for (int j =0; j< pop.size(); ++j) {
		output.push_back(pop[j].getAdults());
	}
	
	return output;
	
}

