/*
 *  population.h
 *  Cameroon
 *
 *  Created by Rafael Guerrero on 3/30/09.
 *  
 *
 */
#ifndef _population
#define population

#include<vector>
using std::vector;

struct Genotype{		// the struct GENOTYPE holds three values (type double). They are the frequencies
						// of states of one inversion locus.
						// It is intended for use inside vectors of frequencies (larvae, zygote...)
	double SS;			// frequency of Standard Homokaryotype	
	double SI;			// frequency of Standard-Inverted heterokaryotype 	
	double II;			// frequency of Inverted Homokaryotype	

    friend std::ostream& operator<<(std::ostream& os, const Genotype& g);

};

class Population {

	friend bool migration(vector<Population>& pop, const vector< double >& mig, double epsilon);
	friend bool migration(vector<Population>& pop,  const vector< double >& mig,vector<int>& mig_points, double lattice_h, double epsilon);
	friend Genotype migKernel(vector<Population>& pop, const vector< double >& mig, double lattice_h, vector<int> indx, bool isLarvae );

public:
	Population (const Genotype & initfreq);
	~Population();
	void goVirgins(const Genotype & W);
	void goZygotes(const double& F );
    void goZygotes(const vector<double> & Fd );
	Genotype getVirgins();
	Genotype getZygotes();
	Genotype getAdults();
	Genotype getLarvae();

private:
	int Nloci;				// Number of inversion sites
							
	Genotype virgins;		// frequencies of genotypes in virgins in present generation
	Genotype zygotes;		// frequencies of genotypes in zygotes in present generation
	Genotype adults;		// frequencies of genotypes in adults in present generation
	Genotype larvae;		// frequencies of genotypes in larvae in next generation (t+1)
	double MeanW;			// vector of mean fitness in locus (used in equation for virgins)
};

vector< Genotype > simulate(vector<Population> pop, 
                            const vector< double >& mig, 
                            const vector<Genotype> & W, 
                            const double & F, 
                            double epsilon, 
                            int maxGens);

vector< Genotype > simulate(vector<Population> pop, 
                            const vector< double >& mig, 
                            const vector<Genotype> & W, 
                            const vector<double >& F, 
                            double epsilon, 
                            int maxGens);

void simulate(vector<Population>& pop, 
              const vector< double >& mig,
              vector<int>& mig_points,
              const vector<Genotype> & W, 
              const double & F, 
              double & epsilon, 
              int & maxGens,
              int & s_h,
              int & h_r,
              double lattice_h  );

#endif